/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 * @defgroup blinky_example_main main.c
 * @{
 * @ingroup blinky_example_freertos
 *
 * @brief Blinky FreeRTOS Example Application main file.
 *
 * This file contains the source code for a sample application using FreeRTOS to blink LEDs.
 *
 */

#include <stdbool.h>
#include <stdint.h>

#include "FreeRTOS.h"
#include "task.h"
#include "timers.h"
#include "bsp.h"
#include "nordic_common.h"
#include "nrf_drv_clock.h"
#include "sdk_errors.h"
#include "app_error.h"
#include "nrf_drv_saadc.h"
   
#include "SEGGER_RTT.h"

#if LEDS_NUMBER <= 2
#error "Board is not equipped with enough amount of LEDs"
#endif

#define TASK_DELAY        1000           /**< Task delay. Delays a LED0 task for 200 ms */
   
/* VREF calculations
*  VREF = 0.6 / (1.0 / 6.0 )
*
*  ASSUMES all channels are set to use the internal reference with a gain of 1/6
*
*/
#define VREF (float)( 3.6 )

#define ADC_MAX_COUNTS (float)( 4095.0 )

/* VCC Calculations
 * VCC = (VCCAN)
 * VCC = ( counts / 4095 ) * VREF
 */
#define VADC_VDD_TO_VOLTS( counts ) ( ( (float)counts / ADC_MAX_COUNTS) * VREF )

/** Defines the default ADC channel configuration */
#define ADC_DEFAULT_CHANNEL_CONFIG_SE(input)           \
{                                                      \
    .resistor_p = NRF_SAADC_RESISTOR_DISABLED,         \
    .resistor_n = NRF_SAADC_RESISTOR_DISABLED,         \
    .gain       = NRF_SAADC_GAIN1_6,                   \
    .reference  = NRF_SAADC_REFERENCE_INTERNAL,        \
    .acq_time   = NRF_SAADC_ACQTIME_10US,              \
    .mode       = NRF_SAADC_MODE_SINGLE_ENDED,         \
    .pin_p      = (nrf_saadc_input_t)(input),          \
    .pin_n      = NRF_SAADC_INPUT_DISABLED             \
}
   
/* private type ------------------------------------------------------------- */
/** A structure to capture an individual channel config */
typedef enum
{
  AIN0,
  AIN1,
  AIN2,
  AIN4,
  ADC_INPUT_VDD,
  ADC_INPUT_END
} adcInput_t;

typedef struct
{
  uint8_t                 channel;
  nrf_saadc_input_t       input;
  nrf_saadc_gain_t        gain;
  nrf_saadc_reference_t   reference;
  nrf_saadc_acqtime_t     acqTime;
} AdcChannelConfig_t;

/* private constants--------------------------------------------------------- */
static const AdcChannelConfig_t  _channelConfigData [ADC_INPUT_END] =
{
  {0, NRF_SAADC_INPUT_AIN0,  NRF_SAADC_GAIN1_6, NRF_SAADC_REFERENCE_INTERNAL, NRF_SAADC_ACQTIME_40US },
  {1, NRF_SAADC_INPUT_AIN1,  NRF_SAADC_GAIN1_6, NRF_SAADC_REFERENCE_INTERNAL, NRF_SAADC_ACQTIME_40US },
  {2, NRF_SAADC_INPUT_AIN2,  NRF_SAADC_GAIN1_6, NRF_SAADC_REFERENCE_INTERNAL, NRF_SAADC_ACQTIME_40US },
  {3, NRF_SAADC_INPUT_AIN4,  NRF_SAADC_GAIN1_6, NRF_SAADC_REFERENCE_INTERNAL, NRF_SAADC_ACQTIME_40US },
  {4, NRF_SAADC_INPUT_VDD,   NRF_SAADC_GAIN1_6, NRF_SAADC_REFERENCE_INTERNAL, NRF_SAADC_ACQTIME_40US },
};   

TaskHandle_t  led_toggle_task_handle;   /**< Reference to LED0 toggling FreeRTOS task. */

static void AppHal_Pwr_AdcIsrCallback( nrf_drv_saadc_evt_t const *p_event )
{
  //do nothing since the ADC will block for each sample
}

static void AppHal_Pwr_AdcInitInput( adcInput_t input )
{
  ret_code_t errCode;
  nrf_saadc_channel_config_t channelConfig = ADC_DEFAULT_CHANNEL_CONFIG_SE(input);

  channelConfig.pin_p     = _channelConfigData[input].input;
  channelConfig.gain      = _channelConfigData[input].gain;
  channelConfig.reference = _channelConfigData[input].reference;
  channelConfig.acq_time  = _channelConfigData[input].acqTime;

  errCode = nrf_drv_saadc_channel_init(_channelConfigData[input].channel, &channelConfig);
  APP_ERROR_CHECK(errCode);
}

static nrf_saadc_value_t AppHal_Pwr_AdcBlockingSample( adcInput_t input )
{
  nrf_saadc_value_t analogSample = 0;
  uint32_t errCode;

  errCode = nrf_drv_saadc_sample_convert( _channelConfigData[input].channel, &analogSample );
  APP_ERROR_CHECK( errCode );

  //handle a negative values from the ADC
  if( analogSample < 0 )
  {
    analogSample = 0;
  }

  return analogSample;
}



/**@brief LED0 task entry function.
 *
 * @param[in] pvParameter   Pointer that will be used as the parameter for the task.
 */
static void led_toggle_task_function (void * pvParameter)
{     
    nrf_saadc_value_t sample;
    char str[64];
    UNUSED_PARAMETER(pvParameter);
    
    
    while (true)
    {
        bsp_board_led_invert(BSP_BOARD_LED_0);
        
        sample = AppHal_Pwr_AdcBlockingSample( AIN2 );
        
        sprintf( str, "Counts %f\n", VADC_VDD_TO_VOLTS(sample) );
        SEGGER_RTT_WriteString( 0, str );

        /* Delay a task for a given number of ticks */
        vTaskDelay(TASK_DELAY);

        /* Tasks must be implemented to never return... */
    }
}

int main(void)
{
    ret_code_t err_code;
    nrf_drv_saadc_config_t defaultConfig = NRF_DRV_SAADC_DEFAULT_CONFIG;

    /* Initialize clock driver for better time accuracy in FREERTOS */
    err_code = nrf_drv_clock_init();
    APP_ERROR_CHECK(err_code);

    /* Configure LED-pins as outputs */
    bsp_board_leds_init();
    bsp_board_leds_off();
    
    // Initialize and configure ADC
    err_code = nrf_drv_saadc_init(&defaultConfig, AppHal_Pwr_AdcIsrCallback);
    APP_ERROR_CHECK(err_code);
    
    /* configure required ADC channel */
    for( uint8_t i = 0; i < ADC_INPUT_END; i++ )
    {
        AppHal_Pwr_AdcInitInput( (adcInput_t)i );
    }
    
    /* Create task for LED0 blinking with priority set to 2 */
    UNUSED_VARIABLE(xTaskCreate(led_toggle_task_function, "LED0", configMINIMAL_STACK_SIZE + 200, NULL, 2, &led_toggle_task_handle));

    /* Activate deep sleep mode */
    SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;

    /* Start FreeRTOS scheduler. */
    vTaskStartScheduler();

    while (true)
    {
        /* FreeRTOS should not be here... FreeRTOS goes back to the start of stack
         * in vTaskStartScheduler function. */
    }
}

/**
 *@}
 **/
